/* Simple Database INC by Sho0ter */

#if !defined _sqlx_included
	#include <sqlx>
#endif

new Handle:dbconnect, Handle:dbtuple, dbavalible
new tempdata[4][128]

stock db_init(const host[], const db[], const user[], const pass[])
{
	new dberror[256], dberrnum
	if(strlen(host))
	{
		dbtuple = SQL_MakeDbTuple(host, user, pass, db)
		copy(tempdata[0], 127, host)
		copy(tempdata[1], 127, db)
		copy(tempdata[2], 127, user)
		copy(tempdata[3], 127, pass)
	}
	else
	{
		dbtuple = SQL_MakeDbTuple(tempdata[0], tempdata[2], tempdata[3], tempdata[1])
	}
	dbconnect = SQL_Connect(dbtuple, dberrnum, dberror, charsmax(dberror))
	if(!dbconnect)
	{
		log_amx("Database connection failed: [%d] %s", dberrnum, dberror)
		return 0
	}
	dbavalible = 1
	return 1
}

stock db_send(const text[])
{
	new Handle:query = SQL_PrepareQuery(dbconnect, text)
	if(!SQL_Execute(query))
	{
		db_error(query, text)
		return 0
	}
	SQL_FreeHandle(query)
	return 1
}

stock db_thread(const text[])
{
	if(!db_is_avalible())
	{
		return 0
	}
	SQL_ThreadQuery(dbtuple, "db_thread_result", text)
	return 1
}

public db_thread_result(failstate, Handle:query, const error[], errnum, const data[], size)
{
	if(failstate)
	{
		new text[256]
		SQL_GetQueryString(query, text, charsmax(text))
		db_thread_error(query, text, error, errnum, failstate)
	}
	else
	{
		SQL_FreeHandle(query)
		return 1
	}
	return 0
}

stock db_close()
{
	if(dbtuple)
	{
		SQL_FreeHandle(dbtuple)
	}
	if(dbconnect)
	{
		SQL_FreeHandle(dbconnect);
	}
	dbavalible = 0
	return 1
}

stock db_error(Handle:query, const text[])
{
	new error[256]
	new errornum = SQL_QueryError(query, error, charsmax(error))
	log_amx("Threaded query error!")
	log_amx("Message: %s (%d)", error, errornum)
	log_amx("Query statement: %s ", text)
	SQL_FreeHandle(query)
	if(errornum == 2006)
	{
		log_amx("Attempting to re-establish connection to MySQL server")
		db_close()
		db_init("", "", "", "")
		return 1
	}
	return 0
}

stock db_thread_error(Handle:query, const text[], const error[], errornum, failstate)
{
	log_amx("Threaded query error!")
	log_amx("Message: %s (%d)", error, errornum)
	log_amx("Query statement: %s ", text)
	if (failstate == TQUERY_CONNECT_FAILED )
	{	
		log_amx("Fail state: Connection Failed")
	}
	else if(failstate == TQUERY_QUERY_FAILED)
	{
		log_amx("Fail state: Query Failed")
	}
	SQL_FreeHandle(query)
	return 1
}

stock db_is_avalible()
{
	return (dbavalible && dbconnect)
}